# -*- coding: utf-8 -*-
"""
Created on Wed Jun 21 23:35:42 2023

@author: eirik
"""

import pandas as pd
import math



# Read timeseries from Excel
df_ts = pd.read_excel('results.xlsx', sheet_name='Data')
df = pd.read_excel('results.xlsx', sheet_name='Input')

# Create a dictionary with variable names as keys and their corresponding values as values
fixed_values_constants = {row['Variable']: row['Value'] for _, row in df.iterrows()}

# Fixed values
A_p = fixed_values_constants['A_p']
A_s = fixed_values_constants['A_s']
A_vb = fixed_values_constants['A_vb']
A_gs = fixed_values_constants['A_gs']
mu = fixed_values_constants['mu']
F_f = fixed_values_constants['F_f']
P_w = fixed_values_constants['P_w']
L = fixed_values_constants['L']
eff = fixed_values_constants['eff']
G_R = fixed_values_constants['G_R']
E_1 = fixed_values_constants['E_1']
E_2 = fixed_values_constants['E_2']
k = fixed_values_constants['k']
n = fixed_values_constants['n']
rpm_open = fixed_values_constants['rpm_open']
rpm_close = fixed_values_constants['rpm_close']
# Create lists to store output
timestamps = []
Actuator_force = []
valve_req = []
states = []
cycle = []
WHSIP_values = []
dP_values = []
positions = []
rpms = []
motor_torque = []


for idx, row in df_ts.iterrows():
    
    # Uses following input data:
    WHSIP = row['WHSIP [psi]']
    position = row['Position [mm]']
    state = row['State']
    power = row['Power [HP]']
    dP = row['dP [psi]']
    position = row['Position [mm]']
    rpm_val = row['RPM']
    
    # Required force estimation for valve states
    F_1 = WHSIP * mu * (2*A_gs + A_vb)
    F_2 = F_f
    F_3 = P_w * (A_p - A_s)
    F_4 = WHSIP * A_s
   

    
    # Loops through the state data from the "Data sheet" to check whick state to calculate for
    if state == 'run_close':
        F = -F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_close)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_close ** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L
        
    elif state == 'crack_close':
        F = -F_1 - F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_close)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_close ** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L
    
            
    elif state == 'crack_open':
        F = F_1 + F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_open)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_open ** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L
        
    elif state == 'run_open':
        F = F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_open)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_open** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L
        
    elif state == 'stay_closed':
        F = F_1 + F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_open)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_open ** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L
    else:
        state = 'stay_open'     
        F = -F_2 + F_3 + F_4
        tau_motor = power * E_1 * 5252 / (rpm_close)
        tau_gear = G_R * tau_motor * E_2/(1 + k * (rpm_close ** (n-1)))
        force_out = tau_gear*2*math.pi*eff/L        
       
    
    
        
        

    # Append to output lists
    timestamps.append(row['Timestamps [s]'])
    Actuator_force.append(force_out)
    valve_req.append(F)
    states.append(state)
    cycle.append(row['Cycle'])
    WHSIP_values.append(WHSIP)
    dP_values.append(dP)
    positions.append(position)
    rpms.append(rpm_val)
    motor_torque.append(tau_motor)
# Create DataFrame from output lists
df_force = pd.DataFrame({'Timestamps [s]': timestamps, 'Cycle': cycle, 'State': states, 'WHSIP [psi]': WHSIP_values, 'dP [psi]': dP_values, 'Position [mm]': positions, 'RPM': rpms, 'Motor Torque [lbf*ft]': motor_torque, 'Est. Actuator output [lbf]': Actuator_force, 'Est. Valve Req. [lbf]': valve_req})

# Write the updated DataFrame back to the Excel file
try:
    with pd.ExcelWriter('results.xlsx', mode='a', engine='openpyxl', if_sheet_exists='overlay') as writer:
        df_force.to_excel(writer, sheet_name='Digital Twin', index=False)
    
    
except FileNotFoundError:
    with pd.ExcelWriter('results.xlsx', mode='w', engine='openpyxl') as writer:
        df_force.to_excel(writer, sheet_name='Digital Twin', index=False)
        
        
        
        

